uniform vec3    BrickColor, MortarColor;
uniform vec2    BrickSize;
uniform vec2    BrickPct;
uniform float   ObjScale;
varying vec3    MCposition;
varying float   LightIntensity;
varying vec3    normal;
float           BricksInRow;
float           shiftx = 0.0;

//-----------------------------------------------------------------
void brick_func(vec2 p, vec3 sp, out vec3 color);
 
//-----------------------------------------------------------------
void main(void)
{
    vec2 position;
    vec3 smcpos;
    vec3 color;
    smcpos = MCposition * ObjScale; 
    position = smcpos.xy / BrickSize;
    float bic = ObjScale / BrickSize.y;

    // top and bottom row are mortar
    if( abs(smcpos.y) / BrickSize.y > floor(bic))
        color = MortarColor;
    else 
        brick_func(position,smcpos,color);

    color *= LightIntensity;
    gl_FragColor = vec4( color, 1.0);
}

//-----------------------------------------------------------------
void brick_func(vec2 p, vec3 sp,out vec3 color)
{
    vec2 useBrick;
    BricksInRow = ObjScale / BrickSize.x;
    float bir = floor(BricksInRow);
    vec2 mpct = vec2(1.0,1.0) - BrickPct;
    float fbir = fract(BricksInRow);

    // modify bir if we have an even division or more than half left over    
    if( fbir == 0.0 )
        bir -= 1.0;
    else if( fbir > .5 )
        bir += 1.0;
 
    if( normal.x == 0.0 ) // front and back faces
    {   
        if( fract(p.y * 0.5 ) > 0.5 ) // brick centered
            shiftx = .5 * BrickPct.x;
    }
    else if( normal.z == 0.0 ) // side faces
    {
        // use z coordinates instead
        p.x = sp.z / BrickSize.x;
        sp.x = sp.z;

        if( fract(p.y * 0.5 ) < 0.5 ) // brick centered
            shiftx = .5 * BrickPct.x;
    }
    
    p.x += shiftx;
    

    // shift so brick color gets centered 
    p -= -0.5 * mpct; 
    p = fract(p);
    p = abs( p - vec2(.5, mpct.y)); 
    
    // little cheat !!!!!!! forces spaces on edge to be brick
    if( (abs(sp.x) / BrickSize.x) > bir)
    {
        p.x = 1.0;
    }
    

    useBrick = smoothstep(mpct * .5, mpct, p);

    color = mix(MortarColor, BrickColor, useBrick.x * useBrick.y);
}
