uniform float   Time;
uniform samplerCube     CubeMapTex;

const float     xdiv = 10.0;
const float     zdiv = 8.0;
const float     magx = 1.6;
const float     magz = 1.0;

varying vec3 normal, lightDir, halfVector, eyeDir;
varying vec4 diffuse, ambient;

void main(void)
{
    float h1, h2;
    vec3 v1, v2;

    // determine current position
    h1 = sin((gl_Vertex.z / zdiv) + Time) * magz;
    h2 = sin((gl_Vertex.x / xdiv) + (Time)) * magx +
         sin((gl_Vertex.x / 5) + (Time - 4));
    h1 = h1 + h2;

    gl_Position = gl_ModelViewProjectionMatrix * 
        vec4(gl_Vertex.x, gl_Vertex.y + h1, gl_Vertex.z,1);

    // get our two slopes in xy and zy
    // d( sin(u) ) = cos(u)d(u)
    v1 = vec3(  0.0, 
                cos((gl_Vertex.z / zdiv) + Time) * magz / zdiv,
                1.0);    
    v2 = vec3(  1.0, 
                cos((gl_Vertex.x / xdiv) + Time) * magx / xdiv +
                cos((gl_Vertex.x / 5.0) + (Time - 4.0)) / 5.0,
                0.0);   

    // find our normal
    // we need derivatives of pos and pos2
    normal = normalize(cross(v1,v2));
   
    eyeDir = vec3( gl_ModelViewMatrix * gl_Vertex); 
    normal = normalize(gl_NormalMatrix * normal);
    lightDir = normalize(vec3(gl_LightSource[1].position));
    halfVector = normalize(gl_LightSource[1].halfVector.xyz);

    diffuse = gl_FrontMaterial.diffuse * gl_LightSource[1].diffuse;
    ambient = gl_FrontMaterial.ambient * gl_LightSource[1].ambient;
    ambient += gl_LightModel.ambient * gl_FrontMaterial.ambient;

}

