/******************************************************************************
 * CoreWall / Corelyzer - An Initial Core Description Tool
 * Copyright (C) 2005 Arun Rao, Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This software is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this software; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern "C" {
#include <jpeglib.h>
};

/*

 * <setjmp.h> is used for the optional error recovery mechanism shown in

 * the second part of the example.

 */



#include <setjmp.h>
#include "jpegimghandler.h"

#define DEBUG_JPEG_LOAD 0

//-------------------------------------------
struct my_error_mgr {

  struct jpeg_error_mgr pub;	/* "public" fields */



  jmp_buf setjmp_buffer;	/* for return to caller */

};
typedef struct my_error_mgr * my_error_ptr;

/*

 * Here's the routine that will replace the standard error_exit method:

 */


//-------------------------------------------

METHODDEF(void)

my_error_exit (j_common_ptr cinfo)

{

  /* cinfo->err really points to a my_error_mgr struct, so coerce pointer */

  my_error_ptr myerr = (my_error_ptr) cinfo->err;



  /* Always display the message. */

  /* We could postpone this until after returning, if we chose. */

  (*cinfo->err->output_message) (cinfo);



  /* Return control to the setjmp point */

  longjmp(myerr->setjmp_buffer, 1);

}

//-------------------------------------------
DLL_EXPORT JPEGImageHandler::JPEGImageHandler()
{
    ImageHandler::ImageHandler();
}

DLL_EXPORT JPEGImageHandler::~JPEGImageHandler()
{

}


DLL_EXPORT bool JPEGImageHandler::Load( char* filename )
{
    if( filename == NULL )
        return false;
        
    if( !strstr(filename,".JPEG") && !strstr(filename,".jpeg")
            && !strstr(filename,".JPG") && !strstr(filename,".jpg"))
    {
        return false;
    }
   
   	sourceFile = new char[ strlen(filename) + 1];
	strcpy(sourceFile,filename);

    jpeg_decompress_struct cinfo;
    my_error_mgr jerr;
    
    FILE* jpgFile = fopen( filename, "rb" );
    if( jpgFile == NULL )
    {
        printf("Cannot open %s\n", filename);
        return false;
    }
    
    cinfo.err = jpeg_std_error(&jerr.pub);

    jerr.pub.error_exit = my_error_exit;

    /* Establish the setjmp return context for my_error_exit to use. */

    if (setjmp(jerr.setjmp_buffer)) {

        /* If we get here, the JPEG code has signaled an error.

        * We need to clean up the JPEG object, close the input file, and return.

        */

        jpeg_destroy_decompress(&cinfo);

        fclose(jpgFile);

        return false;

    }
    
    //----------------------------------------------------
    //JSAMPARRAY buffer;
    char* buffer = NULL;
    int row_stride;
    //----------------------------------------------------
#if DEBUG_JPEG_LOAD    
    printf("creating decompress object\n");
#endif    
    //initialize the jpeg decompress object
    jpeg_create_decompress( &cinfo );
    
#if DEBUG_JPEG_LOAD    
    printf("specifiying data source\n");
#endif        
    //specify data source
    jpeg_stdio_src( &cinfo, jpgFile );
    
#if DEBUG_JPEG_LOAD    
    printf("reading file headers/parameters\n");
#endif    
    //read file parameters
    jpeg_read_header( &cinfo, TRUE );
    
#if DEBUG_JPEG_LOAD    
    printf("starting decompressor\n");
#endif    
    //start decompressor
    jpeg_start_decompress( &cinfo );
    //----------------------------------------------------
    
    //determine the color space
    int components = 0;
    switch( cinfo.out_color_space )
    {
        case JCS_RGB:
            components = 3;
            pixelFormat = RGB;
#if DEBUG_JPEG_LOAD    
    printf("RGB Color Space\n");
#endif    
            break;
        default:
            jpeg_finish_decompress( &cinfo );
            jpeg_destroy_decompress( &cinfo );
            fclose( jpgFile );
#if DEBUG_JPEG_LOAD    
    printf("Unsupported Color Space\n");
#endif    
            return false;
            break;
    };
    
    //image width and height
    width = cinfo.output_width;
    height = cinfo.output_height;
#if DEBUG_JPEG_LOAD    
    printf("Width %d, Height %d\n", width, height);
#endif    

    //----------------------------------------------------
        
    
    /* JSAMPLEs per row in output buffer */
    row_stride = cinfo.output_width * cinfo.output_components;
    //fill in the pixels!
    if( pixels )
        delete [] pixels;
    pixels = new char[ row_stride * height ];
    
    /* Make a one-row-high sample array that 
    will go away when done with image */
    buffer = new char[ row_stride ];
    
#if DEBUG_JPEG_LOAD    
    printf("Grabbing pixels! Mem Start %d, Row Stride: %d\n", 
        pixels, row_stride);
#endif    

    while( cinfo.output_scanline < cinfo.output_height )
    {
#if DEBUG_JPEG_LOAD
    fprintf(stderr," row: %d", cinfo.output_scanline);
    fflush(stderr);
#endif    
        jpeg_read_scanlines( &cinfo, (JSAMPLE**)&buffer, 1);
        memcpy( pixels + ((cinfo.output_scanline - 1) * row_stride),
                buffer, row_stride);
#if DEBUG_JPEG_LOAD
    fprintf(stderr," copied pixels\n");
#endif

    }

#if DEBUG_JPEG_LOAD
    printf("Done Copying... deleting temp row buffer\n");
#endif    
    delete [] buffer;
        		
#if DEBUG_JPEG_LOAD
    printf("Getting density/resolution information\n");
#endif
    //----------------------------------------------------
    //set resolution information if possible
    switch( cinfo.density_unit )
    {
    case 0:
        resFormat = UNDEFINED_FORMAT;
#if DEBUG_JPEG_LOAD
    printf("Undefined Resolution Format! %d\n", resFormat);
#endif
        break;
    case 1:
        resFormat = DPI;
#if DEBUG_JPEG_LOAD
    printf("Resolution Format: Dots per inch!\n");
#endif
        break;
    case 2:
        resFormat = DPCM;
#if DEBUG_JPEG_LOAD
    printf("Resolution Format: Dots per cm!\n");
#endif
        break;
    };
    
    resH = cinfo.X_density;
	resV = cinfo.Y_density;

#if DEBUG_JPEG_LOAD
	printf("Horizontal Resolution: %f\nVertical Resolution: %f", 
        float( resH ), float( resV ) ) ;/// float( cinfo.Y_density) );
    printf("Finishing decompress and destroying decompress!\n");
#endif
    
    jpeg_finish_decompress( &cinfo );
    jpeg_destroy_decompress( &cinfo );
   	
    fclose( jpgFile );
   
   	
    return true;
}
