#include "cpgl.h"
#include "trimodel.h"
#include "jpegimghandler.h"
#include <math.h>
#include <time.h>
#include <unistd.h>

//-----------------------------------
SDL_Surface* gDrawSurface = NULL;
int width = 800;
int height = 600;
//-----------------------------------
// Update Function Pointer and Variables Updated Each Call
void    (*updatefn)(double)    = NULL;
float   Rot                    = 0;
GLint   UniTime[3]             = { -1, -1, -1};
GLint   UniRot[3]              = { -1, -1, -1};
float   TotalTime              = 0;
GLfloat LightPos[4]            = { 0, 15, 30, 0};
GLfloat LightVel[3]            = { 0, 0, 0 };
GLfloat CamPos[3]              = { 0, 10, 55 };
GLfloat CamVel[3]              = { 0, 0, 0 };
bool    Rotate                 = true;

//----------------------------------
// Model objects
TriModel* CowMod               = NULL;
TriModel* CubeMod              = NULL;
GLUquadricObj *GluQuadric      = NULL;

//----------------------------------
GLuint CubeMapTexId;
#define POSX 0
#define POSY 1
#define POSZ 2
#define NEGX 3
#define NEGY 4
#define NEGZ 5

char  CubeMapFiles[6][45] = {
    "textures/cubemaps/sea/posx.jpg",
    "textures/cubemaps/sea/posy.jpg",
    "textures/cubemaps/sea/posz.jpg",
    "textures/cubemaps/sea/negx.jpg",
    "textures/cubemaps/sea/negy.jpg",
    "textures/cubemaps/sea/negz.jpg"} ;

GLenum CubeMapTarget[6] = {
    GL_TEXTURE_CUBE_MAP_POSITIVE_X,
    GL_TEXTURE_CUBE_MAP_POSITIVE_Y,
    GL_TEXTURE_CUBE_MAP_POSITIVE_Z,
    GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
    GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
    GL_TEXTURE_CUBE_MAP_NEGATIVE_Z};

//----------------------------------

//----------------------------------
// waves shaders
GLhandleARB WavePrg, WaveVS, WaveFS;
#define WATER_BOX_SCALE 120   
#define WATER_PT_INTERVAL 1   // interval of points calculated on the water


//----------------------------------
// Brick Parameters
GLfloat BrickColor[3] = { 0.8, 0.0, 0.0 };
GLfloat MortarColor[3] = { 0.7, 0.7, 0.7};
GLfloat BrickSize[2] = { 0.9, 0.4};
GLfloat BrickPct[2] = { 0.9, 0.8 };
GLfloat CubeScale = 4.0;

//-----------------------------------
// Function prototypes
char* GetShaderSource(char* filename);
void SetupShaders(char* vert, char* frag,GLhandleARB& vs,GLhandleARB& fs,
    GLhandleARB& prg);
void DrawScene();
void ProcessKeyDown(SDL_Event* e);
void ProcessKeyUp(SDL_Event* e);
void InfLoop();
void Update(double dt);
void CreateQuadric();
void InitLight();
void SetupSDL();
void SetupGLProcs();
void ReadBrickParams();
void SetupCubeEnv();

#if !defined(__APPLE__) && !defined(_WIN32)

// SHADER FUNCTIONS
PFNGLCREATEPROGRAMOBJECTARBPROC     glCreateProgramObjectARB = NULL;
PFNGLCREATESHADEROBJECTARBPROC      glCreateShaderObjectARB = NULL;
PFNGLSHADERSOURCEARBPROC            glShaderSourceARB = NULL;
PFNGLCOMPILESHADERARBPROC           glCompileShaderARB = NULL;
PFNGLGETOBJECTPARAMETERIVARBPROC    glGetObjectParameterivARB = NULL;
PFNGLATTACHOBJECTARBPROC            glAttachObjectARB = NULL;
PFNGLGETINFOLOGARBPROC              glGetInfoLogARB = NULL;
PFNGLLINKPROGRAMARBPROC             glLinkProgramARB = NULL;
PFNGLUSEPROGRAMOBJECTARBPROC        glUseProgramObjectARB = NULL;
PFNGLGETUNIFORMLOCATIONARBPROC      glGetUniformLocationARB = NULL;
PFNGLUNIFORM1FARBPROC               glUniform1f = NULL;
PFNGLUNIFORM2FARBPROC               glUniform2f = NULL;
PFNGLUNIFORM3FARBPROC               glUniform3f = NULL;
PFNGLUNIFORM1FVARBPROC              glUniform1fv = NULL;
PFNGLUNIFORM2FVARBPROC              glUniform2fv = NULL;
PFNGLUNIFORM3FVARBPROC              glUniform3fv = NULL;

// MULTITEXTURE FUNCTIONS
PFNGLACTIVETEXTUREPROC              glActiveTexture = NULL;
#endif


//-----------------------------------
int main(int argc, char** argv)
{

    printf("loading models!\n");

    CowMod     = LoadTriModel("models/cow.tri");
    CubeMod    = LoadTriModel("models/cube.tri");

    SetupSDL();
    SetupGLProcs();
    InitLight();
    SetupCubeEnv();

    GluQuadric = gluNewQuadric();
    gluQuadricDrawStyle(GluQuadric,GLU_FILL);
    gluQuadricOrientation(GluQuadric,GLU_INSIDE);
    gluQuadricNormals(GluQuadric,GLU_SMOOTH);
    gluQuadricTexture(GluQuadric,GL_TRUE);

    GLint uni; // generic uniform
    // create shaders
    if(CowMod)
    {    
       // SetupShaders(NULL,NULL,
       SetupShaders("shaders/toon.vert","shaders/toon.frag",
            CowMod->vs,CowMod->fs,CowMod->prg);
        UniTime[2] = glGetUniformLocationARB(CowMod->prg,"Time");
        glUniform1f(UniTime[2],0.0);
        UniRot[1] = glGetUniformLocationARB(CowMod->prg,"Spin");
        glUniform1f(UniRot[1],0.0);
        uni = glGetUniformLocationARB(CowMod->prg,"CubeScale");
        glUniform1f(uni,CubeScale);
    }
    
    if(CubeMod)
    {  
        ReadBrickParams();
        SetupShaders("shaders/brick.vert","shaders/brick.frag",
            CubeMod->vs,CubeMod->fs,CubeMod->prg);
        uni = glGetUniformLocationARB(CubeMod->prg,"BrickColor");
        glUniform3fv(uni,1,BrickColor);
        uni = glGetUniformLocationARB(CubeMod->prg,"MortarColor");
        glUniform3fv(uni,1,MortarColor);
        uni = glGetUniformLocationARB(CubeMod->prg,"BrickSize");
        glUniform2fv(uni,1,BrickSize);
        uni = glGetUniformLocationARB(CubeMod->prg,"BrickPct");
        glUniform2fv(uni,1,BrickPct);
        uni = glGetUniformLocationARB(CubeMod->prg,"ObjScale");
        glUniform1f(uni,CubeScale);
        UniTime[1] = glGetUniformLocationARB(CubeMod->prg,"Time");
        glUniform1f(UniTime[1],0.0);
        UniRot[0] = glGetUniformLocationARB(CubeMod->prg,"Spin");
        glUniform1f(UniRot[0],0.0);
    }

    SetupShaders("shaders/waves.vert","shaders/waves.frag", 
        WaveVS,WaveFS,WavePrg);
    
    UniTime[0] = glGetUniformLocationARB(WavePrg,"Time");
    glUniform1f(UniTime[0],0.0);
    uni = glGetUniformLocationARB(WavePrg,"CubeMapTex");
    glUniform1f(uni,0); 
    
    updatefn = Update;
    InfLoop();
    return 0;
}

//-----------------------------------
// Load Shader Source From File
char* GetShaderSource(char* filename)
{
    char* source;
    FILE* fptr = fopen(filename,"rb");
    if( !fptr )
        return NULL;
    fseek(fptr,0,SEEK_END);
    long length = ftell(fptr);
    source = new char[length + 1];
    memset(source,0,length + 1);
    fseek(fptr,0,SEEK_SET);
    fread(source,sizeof(char),length,fptr);
    fclose(fptr);

    return source;
}

//-----------------------------------
// Setup shaders
void SetupShaders(char* vert, char* frag, GLhandleARB& vs, GLhandleARB& fs,
    GLhandleARB& prg)
{
    char infolog[1024];
    GLint length;
    // get shader source code
    char* vertsource = GetShaderSource(vert);
    char* fragsource = GetShaderSource(frag);
    if(!vertsource && !fragsource )
    {
        fprintf(stderr,"No shader source code!\n");
        return;
    }

    prg = glCreateProgramObjectARB();
    
    // process the vertex shader source
    if( vertsource )
    {    
        vs = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
        glShaderSourceARB(vs,1,(const GLcharARB**)&vertsource,NULL);
        glCompileShaderARB(vs);
       
        // see if it compiled
        GLint compiled;
        glGetObjectParameterivARB(vs,GL_OBJECT_COMPILE_STATUS_ARB,
            &compiled);
        
        if(compiled)
            glAttachObjectARB(prg,vs);
        else
        {
            printf("VERTEX SHADER COMPILE PROBLEM\n");
            glGetInfoLogARB(vs,1024,&length,infolog);
            if( length ) printf("GLSL PROGRAM LOG: %s\n", infolog);
        }
    }
   
    // process the fragment shader source code
    if( fragsource )
    {
        fs = glCreateShaderObjectARB(GL_FRAGMENT_SHADER);
        glShaderSourceARB(fs,1,(const GLcharARB**)&fragsource,NULL);
        glCompileShaderARB(fs);
       
        // see if the fragment shader compiled
        GLint compiled;
        glGetObjectParameterivARB(fs,GL_OBJECT_COMPILE_STATUS_ARB,
            &compiled);
        
        if(compiled)
            glAttachObjectARB(prg,fs);
        else
        {
            printf("FRAG SHADER COMPILE PROBLEM\n");
            glGetInfoLogARB(fs,1024,&length,infolog);
            if( length ) printf("GLSL PROGRAM LOG: %s\n", infolog);
        }
    }

    
    glLinkProgramARB(prg);
    
    glGetInfoLogARB(prg,1024,&length,infolog);
    if( length )
    {
        printf("GLSL PROGRAM LOG: %s\n", infolog );
    }
    
    glUseProgramObjectARB(prg);

    if( fragsource ) delete [] fragsource;
    if( vertsource ) delete [] vertsource;

}

//-----------------------------------
// Draw
void DrawScene()
{
    glClearColor(0,.1,.4,.9);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective( 60, 4.0f / 3.0f, .01, 10000);
    glMatrixMode(GL_MODELVIEW);
 //   gluLookAt( 0, 3, 25, 0, 0, -1, 0, 1, 0);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);
    glDisable(GL_BLEND);
    glLightfv(GL_LIGHT1,GL_POSITION,LightPos);
    
    //glEnable(GL_COLOR_MATERIAL);
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    
    glPushMatrix();
    glTranslatef( -CamPos[0], -CamPos[1], -CamPos[2]);
    
    GLfloat cowdif[4]    = { .8, .8, .6, 1 };
    GLfloat waterdif[4]     = { 0, .1, .4, .7 };
    GLfloat envdif[4]     = { 1, 1, 1, 1 };
    glColorMaterial(GL_FRONT,GL_DIFFUSE);
        
        glEnable(GL_DEPTH_TEST);
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_TEXTURE_CUBE_MAP); 
        glBindTexture(GL_TEXTURE_CUBE_MAP, CubeMapTexId);

        // surrounding cube
        glUseProgramObjectARB(0);
        glMaterialfv(GL_FRONT,GL_DIFFUSE,envdif);
        glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_NORMAL_MAP);
        glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_NORMAL_MAP);
        glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_NORMAL_MAP);
        glEnable(GL_TEXTURE_GEN_S);
        glEnable(GL_TEXTURE_GEN_T);
        glEnable(GL_TEXTURE_GEN_R);
        
        glPushMatrix();
        glRotatef(180,1,0,0);
        gluSphere(GluQuadric,WATER_BOX_SCALE * 1.5,80,25);
        glPopMatrix();
        glDisable(GL_TEXTURE_GEN_S);
        glDisable(GL_TEXTURE_GEN_T);
        glDisable(GL_TEXTURE_GEN_R);
        
        
        // cow and cube don't use samplers in frag shader so no need to unbind
        // textures
        
        // Cow 
        glMaterialfv(GL_FRONT,GL_DIFFUSE,cowdif);
        glPushMatrix();
            DrawTriModel(CowMod);
        glPopMatrix();
        // Cube
        glPushMatrix();
            DrawTriModel(CubeMod);
        glPopMatrix();
        
        // water mesh
        glEnable(GL_BLEND);
        //glDisable(GL_DEPTH_TEST);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
        glUseProgramObjectARB(WavePrg);
        glMaterialfv(GL_FRONT,GL_DIFFUSE,waterdif);
        for(    float i = -WATER_BOX_SCALE; 
                i < WATER_BOX_SCALE; 
                i += WATER_PT_INTERVAL)
        {
            glBegin(GL_QUADS);
            glNormal3f(0,1,0);
            for(    float k = -WATER_BOX_SCALE; 
                    k < WATER_BOX_SCALE; 
                    k += WATER_PT_INTERVAL)
            {
               glVertex3f(i,0,k);
               glVertex3f(i,0,k + WATER_PT_INTERVAL);
               glVertex3f(i + WATER_PT_INTERVAL,0,k + WATER_PT_INTERVAL);
               glVertex3f(i + WATER_PT_INTERVAL,0,k);

            }
            glEnd();
        }
       
    glPopMatrix(); // popping camera position

    glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);

    SDL_GL_SwapBuffers();

}

//-----------------------------------
// Process key up presses
void ProcessKeyUp(SDL_Event* e)
{
    switch( e->key.keysym.sym )
    {
    case SDLK_LEFT:
    case SDLK_RIGHT:
    case SDLK_UP:
    case SDLK_DOWN:
        LightVel[0] = LightVel[1] = LightVel[2] = 0;
        break;
    case SDLK_i:
    case SDLK_k:
    case SDLK_j:
    case SDLK_l:
        CamVel[0] = CamVel[1] = CamVel[2] = 0;
        break;
    }
}

//-----------------------------------
// Process key down presses
void ProcessKeyDown(SDL_Event* e)
{
    switch( e->key.keysym.sym )
    {
    case SDLK_ESCAPE:
        FreeTriModel(CowMod);
        FreeTriModel(CubeMod);
        delete CowMod;
        delete CubeMod;
        exit(0);
        break;
    case SDLK_LEFT:
        LightVel[0] = -10;
        break;
    case SDLK_RIGHT:
        LightVel[0] = 10;
        break;
    case SDLK_UP:
        LightVel[2] = -10;
        break;
    case SDLK_DOWN:
        LightVel[2] = 10;
        break;
    case SDLK_r:
        Rotate ? Rotate = false : Rotate = true;
        break;
    case SDLK_i:
        CamVel[2] = -30;
        break;
    case SDLK_k:
        CamVel[2] = 30;
        break;
    case SDLK_j:
   //     CamVel[3] = -5;
        break;
    case SDLK_l:
   //     CamVel[3] = 5;
        break;
    }

    printf("light velocity: %f, %f, %f\n", LightVel[0], LightVel[1], 
	   LightVel[2]);
}

//-----------------------------------
// Infinite Loop
void InfLoop()
{
    double starttime, endtime;
    starttime = clock();
    endtime = starttime;
    int dt = 0;
    starttime = (double) SDL_GetTicks() / 1000.0; 
    
    //--- infinite loop with event queue processing
    SDL_Event event;
    while(1) 
    {
        while( SDL_PollEvent( &event )) 
        {
            switch( event.type )
            {
            case SDL_KEYDOWN:
                ProcessKeyDown(&event);
                break;
            case SDL_KEYUP:
                ProcessKeyUp(&event);
                break;
            case SDL_QUIT:
                FreeTriModel(CowMod);
                FreeTriModel(CubeMod);
                delete CowMod;
                delete CubeMod;
                exit(0);
                break;
            }

        } // -- while event in queue

       
        //capture frame time
        endtime = (double) SDL_GetTicks() / 1000.0; 
        
        if( endtime - starttime > 1.0f / float(DESIRED_FPS) )
        {
            if( updatefn )
                updatefn(endtime - starttime);
            starttime = (double) SDL_GetTicks() / 1000.0;
            
            // update scene 
            DrawScene();
        }

        
    } // -- infinite loop
}

//-----------------------------------
// homebrew update funciton
void Update(double dt)
{
   
    if( Rotate ) Rot -= 10 * dt;

    for( int i = 0; i < 3; i++)
    {
        LightPos[i] += LightVel[i] * dt;
        CamPos[i] += CamVel[i] * dt;
    }

    TotalTime += dt;

    // update our uniforms
    if( UniTime[0] != -1)
    {
        glUseProgramObjectARB(WavePrg);
        glUniform1f(UniTime[0],TotalTime);
    }
    
    if( CubeMod->prg > 0)
    {
        glUseProgramObjectARB(CubeMod->prg);
        if( UniTime[1] != -1)
            glUniform1f(UniTime[1],TotalTime);
        if( UniRot[0] != -1)
            glUniform1f(UniRot[0], Rot / 180.0 * acos(-1.0));
    }
    
    if( CowMod->prg > 0)
    {
        glUseProgramObjectARB(CowMod->prg);
        if( UniTime[2] != -1)
            glUniform1f(UniTime[2],TotalTime);
        if( UniRot[1] != -1)
            glUniform1f(UniRot[1], Rot / 180.0 * acos(-1.0));
    }
}

//-----------------------------------
// Setup lights
void InitLight()
{
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT1);
    
    GLfloat lightamb[] = { .3, .3, .3, 1 };
    GLfloat lightdif[] = { 1, 1, 1, 1 };
    glShadeModel(GL_SMOOTH);
    glLightfv(GL_LIGHT1,GL_AMBIENT,lightamb);
    glLightfv(GL_LIGHT1,GL_DIFFUSE,lightdif);
    glLightfv(GL_LIGHT1,GL_POSITION,LightPos);
    //glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);


}


//-----------------------------------
// SETUP SDL
void SetupSDL()
{
    // init video system
    const SDL_VideoInfo* vidinfo;
    if( SDL_Init(SDL_INIT_VIDEO) < 0 )
    {
        fprintf(stderr,"Failed to initialize SDL Video!\n");
        exit(1);
    }
    
    // tell system which funciton to process when exit() call is made
    atexit(SDL_Quit);

    // get optimal video settings
    vidinfo = SDL_GetVideoInfo();
    if(!vidinfo)
    {
        fprintf(stderr,"Coudn't get video information!\n%s\n", SDL_GetError());
        exit(1);
    }

    // set opengl attributes
    SDL_GL_SetAttribute(SDL_GL_RED_SIZE,        5);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE,      5);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,       5);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,      32);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,    1);

    // get a framebuffer
    gDrawSurface = SDL_SetVideoMode(width,height,vidinfo->vfmt->BitsPerPixel,
        SDL_OPENGL);

    if( !gDrawSurface )
    {
        fprintf(stderr,"Couldn't set video mode!\n%s\n", SDL_GetError());
        fprintf(stderr,"Trying lower depth size to 24\n");
        SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,      24);
        gDrawSurface = SDL_SetVideoMode(width,height,
            vidinfo->vfmt->BitsPerPixel, SDL_OPENGL);
        if(!gDrawSurface)
        {
            fprintf(stderr,"Couldn't set video mode!\n%s\n", SDL_GetError());
            fprintf(stderr,"Trying lower depth size 16\n");
            SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,      16);
            gDrawSurface = SDL_SetVideoMode(width,height,
                vidinfo->vfmt->BitsPerPixel, SDL_OPENGL);
            if(!gDrawSurface)
            {
                fprintf(stderr,"Couldn't set video mode!\n%s\n", 
                    SDL_GetError());
                exit(1);
            }
        }
    }

    // give the window a name
    SDL_WM_SetCaption("ARUN RAO - CS594 PROJ1", "CS594 PROJ1");
    // set opengl viewport and perspective view
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    glViewport(0,0,width,height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective( 60, 4.0f / 3.0f, .01, 10000);
    glMatrixMode(GL_MODELVIEW);
}

#ifdef __APPLE__
void SetupGLProcs()
{
}

#elif !defined(_WIN32)

void SetupGLProcs()
{
    // SHADER PROCS
    glCreateProgramObjectARB = (PFNGLCREATEPROGRAMOBJECTARBPROC)
        glGetProcAddress("glCreateProgramObjectARB");
    glCreateShaderObjectARB = (PFNGLCREATESHADEROBJECTARBPROC)
        glGetProcAddress("glCreateShaderObjectARB");
    glShaderSourceARB = (PFNGLSHADERSOURCEARBPROC)
        glGetProcAddress("glShaderSourceARB");
    glCompileShaderARB = (PFNGLCOMPILESHADERARBPROC)
        glGetProcAddress("glCompileShaderARB");
    glGetObjectParameterivARB = (PFNGLGETOBJECTPARAMETERIVARBPROC)
        glGetProcAddress("glGetObjectParameterivARB");
    glAttachObjectARB = (PFNGLATTACHOBJECTARBPROC)
        glGetProcAddress("glAttachObjectARB");
    glGetInfoLogARB = (PFNGLGETINFOLOGARBPROC)
        glGetProcAddress("glGetInfoLogARB");
    glLinkProgramARB = (PFNGLLINKPROGRAMARBPROC)
        glGetProcAddress("glLinkProgramARB");
    glUseProgramObjectARB = (PFNGLUSEPROGRAMOBJECTARBPROC)
        glGetProcAddress("glUseProgramObjectARB");
    glGetUniformLocationARB = (PFNGLGETUNIFORMLOCATIONARBPROC)
        glGetProcAddress("glGetUniformLocationARB");
    glUniform1f = (PFNGLUNIFORM1FARBPROC)
        glGetProcAddress("glUniform1fARB");
    glUniform2f = (PFNGLUNIFORM2FARBPROC)
        glGetProcAddress("glUniform2fARB");
    glUniform3f = (PFNGLUNIFORM3FARBPROC)
        glGetProcAddress("glUniform3f"); 
    glUniform1fv = (PFNGLUNIFORM1FVARBPROC)
        glGetProcAddress("glUniform1fvARB");
    glUniform2fv = (PFNGLUNIFORM2FVARBPROC)
        glGetProcAddress("glUniform2fvARB");
    glUniform3fv = (PFNGLUNIFORM3FVARBPROC)
        glGetProcAddress("glUniform3fvARB"); 

    // MULTITEXTURE FUNCTIONS
    glActiveTexture = (PFNGLACTIVETEXTUREPROC)
        glGetProcAddress("glActiveTexture");
}

#endif

void ReadBrickParams()
{
    FILE* fptr = fopen("brickparams.txt","r");
    if( !fptr )
        return;

    fscanf(fptr,"%*s %f %f %f\n", &(BrickColor[0]), &(BrickColor[1]),
        &(BrickColor[2]));
    fscanf(fptr,"%*s %f %f %f\n", &(MortarColor[0]), &(MortarColor[1]),
        &(MortarColor[2]));
    fscanf(fptr,"%*s %f %f\n", &(BrickSize[0]), &(BrickSize[1]));
    fscanf(fptr,"%*s %f %f\n", &(BrickPct[0]), &(BrickPct[1]));
    fscanf(fptr,"%*s %f\n", &CubeScale); 


    fclose(fptr);

}

void SetupCubeEnv()
{
    JPEGImageHandler imghandler; // load up all six first
    
    // build textures
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_TEXTURE_CUBE_MAP);
    glGenTextures(1,&CubeMapTexId);
   

    glBindTexture(GL_TEXTURE_CUBE_MAP, CubeMapTexId);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    GLenum pixelformat;
   
    for( int i = 0; i < 6; i++)
    {
        if(!imghandler.Load(CubeMapFiles[i]))
        {
            printf("Couldn't load %s!!! exiting!!!\n", CubeMapFiles[i]);
            exit(1);
        }
        printf("Pixel Format %d\n", imghandler.GetPixelFormat());

        switch( imghandler.GetPixelFormat())
        {
        case RGB:   pixelformat = GL_RGB; break;
        default:
            printf("Unsupported Pixel Format.  Should Be RGB!!!\n");
            exit(1);
        }

        glTexImage2D(CubeMapTarget[i], 0, GL_RGB, 
            imghandler.GetWidth(), imghandler.GetHeight(), 0, pixelformat, 
            GL_UNSIGNED_BYTE, (const GLubyte*) imghandler.GetPixels());
       
    }

    printf("Successfully loaded cube map textures!!!\n");
}
